@@ -15,10 +15,10 @@ import re
 import xlrd
 from PIL import Image
 import qrcode
-import decimal
+from decimal import Decimal, ROUND_HALF_UP, InvalidOperation, DecimalException
 
 
-__version__ = '2.3'
+__version__ = '2.4'
 
 __all__ = [
     'dc_gen_code', 'dc_records_changed', 'start_end_time', 'start_end_time_ex',
@@ -310,11 +310,19 @@ class JdTools(object):
         return operator.eq(x, y)    # Python 3  True or False
 
     @staticmethod
-    def get_filename(name, excel2010=False):
-        """ 返回 唯一 文件名，name 后面增加 年月日 时分秒 毫秒后缀   excel2010 true-4位后缀文件扩展名"""
+    def get_filename(name, excel2010=False, no_ext=False, no_ms=False):
+        """
+        返回 唯一 文件名，name 后面增加 年月日_时分秒_毫秒 后缀
+        :param name:        文件名前缀
+        :param excel2010:   excel文件后缀格式， True-4位后缀文件扩展名， False - xls
+        :param no_ext:      不带文件名后缀
+        :param no_ms:       文件名不带毫秒
+        :return:    文件名 name + 日期 + 扩展名
+        """
         tm = datetime.datetime.today()
-        time_str = datetime.datetime.strftime(tm, '_%Y%m%d_%H%M%S_%f')
-        return name + time_str + '.xls' + ('x' if excel2010 else '')
+        fmt = '_%Y%m%d_%H%M%S' if no_ms else '_%Y%m%d_%H%M%S_%f'
+        time_str = datetime.datetime.strftime(tm, fmt)
+        return name + time_str + ('' if no_ext else ('.xls' + ('x' if excel2010 else '')))
 
     @staticmethod
     def str(val):
@@ -466,7 +474,7 @@ class JdTools(object):
         :param add_hms: 转换后是否添加时分秒
         :return:
         """
-        if isinstance(s_dt, datetime.datetime) or isinstance(s_dt, datetime.date):
+        if isinstance(s_dt, (datetime.datetime, datetime.date)):
             return s_dt
 
         now = datetime.datetime.today()
@@ -505,11 +513,15 @@ class JdTools(object):
         return date_time
 
     @staticmethod
-    def str_to_time(s: str) -> datetime.time:
+    def str_to_time(s):
         """
         字符串 转 时间。
         s 格式：HH:MM:SS， 格式一点都不能错
         """
+        if not s:
+            return None
+        if isinstance(s, datetime.time):
+            return s
         return datetime.time.fromisoformat(s)
 
     @staticmethod
@@ -839,9 +851,14 @@ class JdTools(object):
 
     @staticmethod
     def compare_time(time1, time2):
-        """ 两个时间段时间是否冲突
-        例：time1 = 08:00-09:00
-            time2 = 08:30-09:30
+        """
+        判断 两个时间段时间是否冲突
+        :param time1:   时间段1, 例如 time1 = '08:00-09:00'
+        :param time2:   时间段2，例如 time2 = '08:30-09:30'
+        :return: 冲突返回 False, 不冲突，返回 True
+
+        例：time1 = '08:00-09:00'
+            time2 = '08:30-09:30'
             返回False
         """
         arr1 = time1.split('-')
@@ -861,23 +878,25 @@ class JdTools(object):
     @staticmethod
     def decimal(number, result=False, pt=2):
         """将number数值转换为decimal类型，支持数值类型：int,str,float
+            改为 四舍五入方式 ROUND_HALF_UP。 默认为 ROUND_HALF_EVEN 四舍六入，五平均（前一位是偶数时进位）
             result = True, 返回  number, 成功标识
             result = False, 返回  number
         """
         success = True
+        exp = Decimal('0.' + pt * '0')
         if number == '' or number is None:      # 容错处理： None, ''(空串), 返回 0
             number = 0
         elif isinstance(number, float):
-            number = decimal.Decimal(JdTools.get_float(number, pt))
+            number = Decimal.from_float(number).quantize(exp, rounding=ROUND_HALF_UP)
         elif isinstance(number, int):
-            number = decimal.Decimal(number)
+            number = Decimal(number)
         elif isinstance(number, str):
             try:
-                number = decimal.Decimal(number).quantize(decimal.Decimal('0.' + pt * '0'))
-            except (decimal.InvalidOperation, decimal.DecimalException):
+                number = Decimal(number).quantize(exp, rounding=ROUND_HALF_UP)
+            except (InvalidOperation, DecimalException):
                 success = False
-        elif isinstance(number, decimal.Decimal):
-            number = number.quantize(decimal.Decimal('0.' + pt * '0'))
+        elif isinstance(number, Decimal):
+            number = number.quantize(exp, rounding=ROUND_HALF_UP)
         else:
             success = False
 
@@ -1457,7 +1476,7 @@ def singleton(cls):
 
 def humanize_date_delta(start, end):
     """
-    将时间差转换为友好格式，如“7天”，“3个月”，“2年”
+    将时间差转换为友好格式，如“7天”，“3个月”，“2年”， end > start，否则返回 空''
     :param start:     起始时间，datetime.datetime 或 datetime.date
     :param end:       结束时间，datetime.datetime 或 datetime.date
     :return:          转换结果，string
@@ -1479,6 +1498,6 @@ def humanize_date_delta(start, end):
     elif 30 <= days < 365:
         months = days // 30
         return f'{months}个月'
-    elif 365 <= days:
+    else:  # days >= 365
         years = days // 365
         return f'{years}年'
